from typing import Optional

import attr
from marshmallow_annotations.ext.attrs import AttrsSchema

@attr.s(auto_attribs=True, kw_only=True)
class Integration:
  name: Optional[str] = None
  short_name: Optional[str] = None
  picture: Optional[str] = None
  owner_id: Optional[str] = None
  owner_name: Optional[str] = None
  region: Optional[str] = None
  connection_url: Optional[str] = None
  host: Optional[str] = None
  account: Optional[str] = None
  database: Optional[str] = None
  warehouse: Optional[str] = None
  project: Optional[str] = None
  port: Optional[int] = None
  credentials: Optional[str] = None
  access_key_name: Optional[str] = None
  access_key: Optional[str] = None
  secret: Optional[str] = None
  user: Optional[str] = None
  password: Optional[str] = None
  deleted: Optional[bool] = False

class IntegrationSchema(AttrsSchema):
  class Meta:
    target = Integration
    register_as_scheme = True


from functools import wraps
from flask import (
    request,
    abort
)
from werkzeug.exceptions import BadRequest
from werkzeug.routing import ValidationError


def validate_json(f):
    @wraps(f)
    def wrapper(*args, **kw):
        try:
            if request.get_json() is None:
                raise BadRequest
        except BadRequest:
            return abort(400, 'No JSON body')
        return f(*args, **kw)

    return wrapper


def validate(schema, json, path=''):
    for key in schema:
        req_type = schema[key]

        if isinstance(req_type, OptionalParam):
            if key not in json:
                continue
            req_type = req_type.get_type()

        if key not in json:
            raise ValidationError(f'Expected {path}{key} be defined!')

        if not isinstance(req_type, dict) and not isinstance(json[key], req_type):
            raise ValidationError(f'Expected {path}{key} to be {req_type.__name__}, was {type(json[key]).__name__}')

        if isinstance(req_type, dict) and not isinstance(json[key], dict):
            raise ValidationError(f'Expected {path}{key} to be dictionary, was {type(json[key]).__name__}')

        if isinstance(req_type, dict):
            validate(req_type, json[key], f'{key}.')
            continue

        # else:
        #  abort(500, f'Malformed server schema - {key}')


def validate_schema(schema):
    def decorator(f):
        @wraps(f)
        def wrapper(*args, **kw):
            try:
                validate(schema, request.json)
            except ValidationError as e:
                return abort(400, str(e) if e else 'Invalid or malformed JSON')
            return f(*args, **kw)

        return wrapper

    return decorator


class OptionalParam:
    def __init__(self, t):
        self._type = t

    def get_type(self):
        return self._type
